package com.ruoyi.asset.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.QueryChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.asset.domain.TbAssetCategory;
import com.ruoyi.asset.mapper.TbAssetCategoryMapper;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.system.mapper.SysDeptMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.asset.mapper.TbAssetMapper;
import com.ruoyi.asset.domain.TbAsset;
import com.ruoyi.asset.service.ITbAssetService;

import javax.annotation.Resource;

/**
 * 资产信息Service业务层处理
 * 
 * @author 原动力
 * @date 2023-03-27
 */
@Service
public class TbAssetServiceImpl extends ServiceImpl<TbAssetMapper, TbAsset> implements ITbAssetService
{
    @Autowired
    private TbAssetMapper tbAssetMapper;

    @Resource
    private SysDeptMapper sysDeptMapper;

    @Resource
    private TbAssetCategoryMapper tbAssetCategoryMapper;

    /**
     * 查询资产信息
     * 
     * @param id 资产信息主键
     * @return 资产信息
     */
    @Override
    public TbAsset selectTbAssetById(Long id)
    {
        return tbAssetMapper.selectTbAssetById(id);
    }

    /**
     * 查询资产信息列表
     * 
     * @param tbAsset 资产信息
     * @return 资产信息
     */
    @Override
    public List<TbAsset> selectTbAssetList(TbAsset tbAsset)
    {
        return tbAssetMapper.selectTbAssetList(tbAsset);
    }

    /**
     * 新增资产信息
     * 
     * @param tbAsset 资产信息
     * @return 结果
     */
    @Override
    public int insertTbAsset(TbAsset tbAsset)
    {
        return tbAssetMapper.insertTbAsset(tbAsset);
    }

    /**
     * 修改资产信息
     * 
     * @param tbAsset 资产信息
     * @return 结果
     */
    @Override
    public int updateTbAsset(TbAsset tbAsset)
    {
        return tbAssetMapper.updateTbAsset(tbAsset);
    }

    /**
     * 批量删除资产信息
     * 
     * @param ids 需要删除的资产信息主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetByIds(Long[] ids)
    {
        return tbAssetMapper.deleteTbAssetByIds(ids);
    }

    /**
     * 删除资产信息信息
     * 
     * @param id 资产信息主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetById(Long id)
    {
        return tbAssetMapper.deleteTbAssetById(id);
    }

    /**
     * 根据条形码查询资产信息
     *
     * @param barCode 资产条形码
     * @return 资产信息
     */
    @Override
    public TbAsset selectTbAssetByBarCode(String barCode) {
        // 判断条形码是否为空
        if (StrUtil.isBlank(barCode)) {
            return null;
        }
        // 查询资产信息
        return query().eq("bar_code", barCode).one();
    }

    @Override
    public boolean batchTbAsset(List<TbAsset> tbAssetList) {
        int success = 0;
        List<SysDept> deptList = sysDeptMapper.allDept();
        List<TbAssetCategory> assetCategoryList = tbAssetCategoryMapper.selectList(null);
        for (TbAsset tbAsset : tbAssetList) {
            if (StrUtil.isNotBlank(tbAsset.getBarCode())) {
                long count = deptList.stream().filter(sysDept -> sysDept.getDeptId().equals(tbAsset.getDeptId())).count();
                List<SysDept> collect = deptList.stream().filter(sysDept -> sysDept.getDeptName().equals(tbAsset.getDeptName())).collect(Collectors.toList());
                SysDept dept = null;
                if (!collect.isEmpty()) {
                    dept = collect.get(0);
                }

                long count1 = assetCategoryList.stream().filter(tbAssetCategory -> tbAssetCategory.getNumber().equals(tbAsset.getCategoryNumber())).count();
                if ((count > 0 || dept != null) && count1 > 0) {
                    if (dept != null) {
                        tbAsset.setDeptId(dept.getDeptId());
                    }
                    int replace = tbAssetMapper.replace(tbAsset);
                    if (replace > 0) {
                        success++;
                    }
                }
            }
        }
        return success != 0;
    }

    @Override
    public List<TbAsset> exportData(TbAsset asset) {
        List<SysDept> deptList = sysDeptMapper.allDept();
        List<TbAsset> assetList = selectTbAssetList(asset);
        for (TbAsset tbAsset : assetList) {
            List<SysDept> collect = deptList.stream().filter(sysDept -> sysDept.getDeptId().equals(tbAsset.getDeptId())).collect(Collectors.toList());
            SysDept sysDept = collect.get(0);
            tbAsset.setDeptName(sysDept.getDeptName());
        }
        return assetList;
    }
}
